searchData={"items":[{"type":"module","doc":"","title":"Evaluator","ref":"Evaluator.html"},{"type":"function","doc":"","title":"Evaluator.run_number_series/0","ref":"Evaluator.html#run_number_series/0"},{"type":"function","doc":"","title":"Evaluator.run_politician/0","ref":"Evaluator.html#run_politician/0"},{"type":"function","doc":"","title":"Evaluator.run_schema/0","ref":"Evaluator.html#run_schema/0"},{"type":"module","doc":"The `Mentor` module facilitates interactions with Large Language Models (LLMs) by managing conversation state, configuring adapters, and validating responses against specified schemas.","title":"Mentor","ref":"Mentor.html"},{"type":"module","doc":"- Initiate and manage chat sessions with various LLM adapters.\n- Configure session parameters, including retry limits and debugging options.\n- Validate LLM responses against predefined schemas to ensure data integrity. Supported schemas include `Ecto` schemas, structs, raw maps, `NimbleOptions`, and `Peri` schemas.\n\n> #### Note {: .warning}\n>\n> For now, until `v0.1.0` only `Ecto` shemas are supported.","title":"Features - Mentor","ref":"Mentor.html#module-features"},{"type":"module","doc":"If a structured output request fails, `mentor` support retries, defaulting to a max number of 3 retries, that can be overwritten.\n\n`mentor` also applies by default, an exponential backoff while to execute the next retry attempt, the backoff calculation, that formula is used:\n\n```\nmin(max_backoff, (base_backoff * 2) ^ retry_count)\n```\n\nExample:\n\n| Attempt | Base Backoff | Max Backoff | Sleep time |\n|---------|--------------|-------------|----------------|\n| 1       | 10           | 30000       | 20 |\n| 2       | 10           | 30000       | 400 |\n| 3       | 10           | 30000       | 8000 |\n| 4       | 10           | 30000       | 30000 |\n| 5       | 10           | 30000       | 30000 |\n\n> considering the default values for `base_backoff` (1s) and `max_backoff` (5s)\n\nThose backoff values can be overwritten by using `configure_backoff/2` function.","title":"Backoff calculation - Mentor","ref":"Mentor.html#module-backoff-calculation"},{"type":"function","doc":"Adds a new message to the conversation history.","title":"Mentor.append_message/2","ref":"Mentor.html#append_message/2"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.\n- `message` - A map representing the message to be added, typically containing:\n  - `:role` - The role of the message sender (e.g., \"user\", \"assistant\", \"system\", \"developer\").\n  - `:content` - The content of the message (e.g. a raw string).","title":"Parameters - Mentor.append_message/2","ref":"Mentor.html#append_message/2-parameters"},{"type":"function","doc":"- An updated `Mentor` struct with the new message appended to the `messages` list.","title":"Returns - Mentor.append_message/2","ref":"Mentor.html#append_message/2-returns"},{"type":"function","doc":"iex> mentor = %Mentor{}\n    iex> message = %{role: \"user\", content: \"Hello, assistant!\"}\n    iex> Mentor.append_message(mentor, message)\n    %Mentor{messages: [%{role: \"user\", content: \"Hello, assistant!\"}]}","title":"Examples - Mentor.append_message/2","ref":"Mentor.html#append_message/2-examples"},{"type":"function","doc":"Completes the interaction by sending the accumulated messages to the LLM adapter and processing the response.","title":"Mentor.complete/1","ref":"Mentor.html#complete/1"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.","title":"Parameters - Mentor.complete/1","ref":"Mentor.html#complete/1-parameters"},{"type":"function","doc":"- `{:ok, result}` on successful completion, where `result` is the validated and processed response.\n- `{:error, reason}` on failure, with `reason` indicating the cause of the error.","title":"Returns - Mentor.complete/1","ref":"Mentor.html#complete/1-returns"},{"type":"function","doc":"iex> mentor = %Mentor{adapter: Mentor.LLM.Adapters.OpenAI, __schema__: MySchema, config: [model: \"gpt-4\"]}\n    iex> Mentor.complete(mentor)\n    {:ok, %MySchema{}}\n\n    iex> mentor = %Mentor{adapter: nil, __schema__: MySchema}\n    iex> Mentor.complete(mentor)\n    {:error, :adapter_not_configured}","title":"Examples - Mentor.complete/1","ref":"Mentor.html#complete/1-examples"},{"type":"function","doc":"Same as `complete/1` but it raises an exception if it fails","title":"Mentor.complete!/1","ref":"Mentor.html#complete!/1"},{"type":"function","doc":"Configures the LLM adapter with the given options.","title":"Mentor.configure_adapter/2","ref":"Mentor.html#configure_adapter/2"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.\n- `config` - A keyword list of configuration options for the adapter.","title":"Parameters - Mentor.configure_adapter/2","ref":"Mentor.html#configure_adapter/2-parameters"},{"type":"function","doc":"- An updated `Mentor` struct with the merged adapter configuration.","title":"Returns - Mentor.configure_adapter/2","ref":"Mentor.html#configure_adapter/2-returns"},{"type":"function","doc":"iex> mentor = %Mentor{config: [model: \"gpt-3.5\"]}\n    iex> new_config = [temperature: 0.7]\n    iex> Mentor.configure_adapter(mentor, new_config)\n    %Mentor{config: [model: \"gpt-3.5\", temperature: 0.7]}","title":"Examples - Mentor.configure_adapter/2","ref":"Mentor.html#configure_adapter/2-examples"},{"type":"function","doc":"Configures the exponential backoff values to be used on retry attempts in case of failed requests.","title":"Mentor.configure_backoff/2","ref":"Mentor.html#configure_backoff/2"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.\n- `config` - A keyword list of configuration options for the backoff.\n  - `:max_backoff` - the max value of the backoff can wait in ms, defaults to 5s\n  - `:base_backoff` - the base value of the backoff can wait in ms, default to 1s","title":"Parameters - Mentor.configure_backoff/2","ref":"Mentor.html#configure_backoff/2-parameters"},{"type":"function","doc":"- An updated `Mentor` struct with the merged bacoff configuration.","title":"Returns - Mentor.configure_backoff/2","ref":"Mentor.html#configure_backoff/2-returns"},{"type":"function","doc":"iex> mentor = %Mentor{config: [model: \"gpt-3.5\"]}\n    iex> backoff = [max_backoff: to_timeout(second: 10)]\n    iex> Mentor.configure_backoff(mentor, backoff)\n    %Mentor{config: [model: \"gpt-3.5\"], timeout: [max_backoff: 10_000, base_backoff: 1_000]}","title":"Examples - Mentor.configure_backoff/2","ref":"Mentor.html#configure_backoff/2-examples"},{"type":"function","doc":"Configures the underlying HTTP client used to make request, with the given options.","title":"Mentor.configure_http_client/3","ref":"Mentor.html#configure_http_client/3"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.\n- `http_client` - The HTTP client to use underlying, needs to implement the `Mentor.LLM.Adapter` behaviour and default to the `Mentor.HTTPClient.Finch`.\n- `config` - A keyword list of configuration options for the underlying HTTP client.","title":"Parameters - Mentor.configure_http_client/3","ref":"Mentor.html#configure_http_client/3-parameters"},{"type":"function","doc":"- An updated `Mentor` struct with the chosen HTTP client and its config.","title":"Returns - Mentor.configure_http_client/3","ref":"Mentor.html#configure_http_client/3-returns"},{"type":"function","doc":"iex> mentor = %Mentor{http_client: Mentor.HTTPClient.Finch, http_config: []}\n    iex> config = [request_timeout: 50_000]\n    iex> Mentor.configure_http_client(mentor, MyReqAdapter, config)\n    %Mentor{http_client: MyReqAdapter, http_config: ^config}\n\n    iex> mentor = %Mentor{http_client: Mentor.HTTPClient.Finch, http_config: []}\n    iex> config = [request_timeout: 50_000]\n    iex> Mentor.configure_http_client(mentor, config)\n    %Mentor{http_client: Mentor.HTTPClient.Finch, http_config: ^config}\n\n    iex> mentor = %Mentor{http_client: Mentor.HTTPClient.Finch, http_config: []}\n    iex> Mentor.configure_http_client(mentor, MyReqAdapter)\n    %Mentor{http_client: MyReqAdapter, http_config: []}","title":"Examples - Mentor.configure_http_client/3","ref":"Mentor.html#configure_http_client/3-examples"},{"type":"function","doc":"Sets the maximum number of retries for validation failures.","title":"Mentor.define_max_retries/2","ref":"Mentor.html#define_max_retries/2"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.\n- `max` - An integer specifying the maximum number of retries.","title":"Parameters - Mentor.define_max_retries/2","ref":"Mentor.html#define_max_retries/2-parameters"},{"type":"function","doc":"- An updated `Mentor` struct with the new `max_retries` value.","title":"Returns - Mentor.define_max_retries/2","ref":"Mentor.html#define_max_retries/2-returns"},{"type":"function","doc":"iex> mentor = %Mentor{max_retries: 3}\n    iex> Mentor.define_max_retries(mentor, 5)\n    %Mentor{max_retries: 5}","title":"Examples - Mentor.define_max_retries/2","ref":"Mentor.html#define_max_retries/2-examples"},{"type":"macro","doc":"","title":"Mentor.is_llm_adapter/1","ref":"Mentor.html#is_llm_adapter/1"},{"type":"type","doc":"","title":"Mentor.message/0","ref":"Mentor.html#t:message/0"},{"type":"function","doc":"Overwrites the initial prompt for the LLM session.","title":"Mentor.overwrite_initial_prompt/2","ref":"Mentor.html#overwrite_initial_prompt/2"},{"type":"function","doc":"- `mentor` - The current `Mentor` struct.\n- `initial_prompt` - A string containing the new initial prompt.","title":"Parameters - Mentor.overwrite_initial_prompt/2","ref":"Mentor.html#overwrite_initial_prompt/2-parameters"},{"type":"function","doc":"- An updated `Mentor` struct with the new initial prompt, overwritten.","title":"Returns - Mentor.overwrite_initial_prompt/2","ref":"Mentor.html#overwrite_initial_prompt/2-returns"},{"type":"function","doc":"iex> mentor = %Mentor{}\n    iex> new_prompt = \"You are a helpful assistant.\"\n    iex> Mentor.overwrite_initial_prompt(mentor, new_prompt)\n    %Mentor{initial_prompt: \"You are a helpful assistant.\"}","title":"Examples - Mentor.overwrite_initial_prompt/2","ref":"Mentor.html#overwrite_initial_prompt/2-examples"},{"type":"function","doc":"Starts a new interaction pipeline based on a schema.","title":"Mentor.start_chat_with!/2","ref":"Mentor.html#start_chat_with!/2"},{"type":"function","doc":"- `adapter` - The LLM adapter module to handle interactions (e.g., `Mentor.LLM.Adapters.OpenAI`).\n- `opts` - A keyword list of options:\n  - `:schema` - The schema module or map defining the expected data structure, required.\n  - `:max_retries` (optional) - The maximum number of retries for validation failures (default: 3).","title":"Parameters - Mentor.start_chat_with!/2","ref":"Mentor.html#start_chat_with!/2-parameters"},{"type":"function","doc":"iex> Mentor.start_chat_with!(Mentor.LLM.Adapters.OpenAI, schema: MySchema)\n    %Mentor{}\n\n    iex> Mentor.start_chat_with!(UnknownLLMAdapter, schema: MySchema)\n    ** (RuntimeError) UnknownLLMAdapter should implement the Mentor.LLM.Adapter behaviour.\n\n    iex> Mentor.start_chat_with!(Mentor.LLM.Adapters.OpenAI, schema: nil)\n    ** (RuntimeError) nil should be a valid schema","title":"Examples - Mentor.start_chat_with!/2","ref":"Mentor.html#start_chat_with!/2-examples"},{"type":"type","doc":"Represents the state of a Mentor session.","title":"Mentor.t/0","ref":"Mentor.html#t:t/0"},{"type":"type","doc":"- `:__schema__` - The schema module or map defining the expected data structure.\n- `:json_schema` - The JSON schema map derived from the schema, used for validation.\n- `:adapter` - The LLM adapter module responsible for handling interactions.\n- `:initial_prompt` - The initial system prompt guiding the LLM's behavior.\n- `:messages` - A list of messages exchanged in the session.\n- `:config` - Configuration options for the adapter.\n- `:max_retries` - The maximum number of retries allowed for validation failures.\n- `:debug` - A boolean flag indicating whether debugging is enabled.\n- `:http_client` - The HTTP Client that implements the `Mentor.HTTPClient.Adapter` behaviour to be used to dispatch HTTP requests to the LLM adapter.\n- `:timeout` - Configures how the timeout backoff should be applied on retries. Check [Backoff calculation to understand it better](#backoff-calculation)\n  - `:max_backoff` - The maximum backoff value in ms, default: `5s`.\n  - `:base_backoff` - The base backoff value in ms, default: `1s`.","title":"Fields - Mentor.t/0","ref":"Mentor.html#t:t/0-fields"},{"type":"module","doc":"Helper module to generate JSON Schema based on Ecto schema.\n\nJSON Schema comes in many flavors and different LLMs have different limitations. Currently, this module aims to implement a schema suitable for [OpenAI structured outputs](https://platform.openai.com/docs/guides/structured-outputs/supported-schemas). This type of JSON schema may not be optimal or compatible with other models. Therefore, it's recommended to use this module as a starting point to generate a schema during development.\n\n> #### Warning {: .warning}\n>\n> For the reasons described above, neither backward compatibility nor compatibility with all built-in adapters are goals of this module.","title":"Mentor.Ecto.JSONSchema","ref":"Mentor.Ecto.JSONSchema.html"},{"type":"function","doc":"Generates a JSON Schema from an Ecto schema.","title":"Mentor.Ecto.JSONSchema.from_ecto_schema/1","ref":"Mentor.Ecto.JSONSchema.html#from_ecto_schema/1"},{"type":"behaviour","doc":"Provides functionality to integrate Ecto schemas with the Mentor framework, ensuring that schemas include comprehensive field documentation.\n\nThis module defines a behaviour that requires implementing a `changeset/2` function and utilizes compile-time hooks to verify that all fields in the schema are documented in the module's `@moduledoc`.","title":"Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html"},{"type":"behaviour","doc":"To use `Mentor.Ecto.Schema` in your Ecto schema module:\n\n    defmodule MyApp.Schema do\n      use Ecto.Schema\n      use Mentor.Ecto.Schema\n\n      import Ecto.Changeset\n\n      @primary_key false\n      embedded_schema do\n        field :name, :string\n        field :age, :integer\n      end\n\n      @impl true\n      def changeset(%__MODULE__{} = source, %{} = attrs) do\n        source\n        |> cast(attrs, [:name, :age])\n        |> validate_required([:name, :age])\n        |> validate_number(:age, less_than: 100, greater_than: 0)\n      end\n    end\n\nEnsure that your module's `@moduledoc` includes a \"Fields\" section documenting each field:\n\n    @moduledoc \"\"\"\n    Schema representing a person.","title":"Usage - Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html#module-usage"},{"type":"behaviour","doc":"- `name`: The name of the person.\n    - `age`: The age of the person.\n    \"\"\"","title":"Fields - Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html#module-fields"},{"type":"behaviour","doc":"If you don't wanna or can't rely on `@moduledoc` to descrive the LLM prompt for your schema, you can alternatively provide a `llm_description/0` callback into you schema module that returns a string that represents the prompt it self, like:\n\n    @impl true\n    def llm_description do\n      \"\"\"","title":"Custom LLM description - Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html#module-custom-llm-description"},{"type":"behaviour","doc":"- `name`: it should be a valid string name for humans\n      - `age`: it should be a reasonable age number for a human being\n      \"\"\"\n    end","title":"Fields - Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html#module-fields"},{"type":"behaviour","doc":"Sometimes you wanna use an Ecto schema field only for internal logic or even have different changesets functions that can cast on different set of fields and for so you would like to avoid to send these fields to the LLM and avoid the strictness of filling the description for these fields in the `@moduledoc`.\n\nIn this case you can pass an additional option while using this module, called `ignored_fields`, passing a list of atoms with the fields names to be ignored, for instance:\n\n    defmodule MyApp.Schema do\n      use Ecto.Schema\n      use Mentor.Ecto.Schema, ignored_fields: [:timestamps]\n\n      import Ecto.Changeset\n\n      @timestamps_opts [inserted_at: :created_at]\n\n      @primary_key false\n      embedded_schema do\n        field :name, :string\n        field :age, :integer\n\n        timestamps()\n      end\n\n      @impl true\n      def changeset(%__MODULE__{} = source, %{} = attrs) do\n        source\n        |> cast(attrs, [:name, :age])\n        |> validate_required([:name, :age])\n        |> validate_number(:age, less_than: 100, greater_than: 0)\n      end\n    end\n\nOne so common use case for this option, as can be seen on the aboce example are the timestamps fields that Ecto generate, so for this special case you can inform `:timestamps` as an ignored field to ignore both `[:inserted_at, :updated_at]`, even if you define custom aliases for it with the `@timestamps_opts` attribute, like `:created_at`.\n\nYou can also pass partial timestamps fields to be ignored, like only ignore `:created_at` or `:updated_at`.\n\n>","title":"Ignored fields - Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html#module-ignored-fields"},{"type":"behaviour","doc":">\n> Defining timestamps aliases with the macro `timestamps/1` inside the schema itself, aren't supported, since i didn't discover how to get this data from on compile time to filter as ignored fields, sou you can either define these options as the attribute as said above, or pass the individual aliases names into the `ignored_fields` options.","title":"Warning {: .warning} - Mentor.Ecto.Schema","ref":"Mentor.Ecto.Schema.html#module-warning-warning"},{"type":"callback","doc":"","title":"Mentor.Ecto.Schema.changeset/2","ref":"Mentor.Ecto.Schema.html#c:changeset/2"},{"type":"callback","doc":"","title":"Mentor.Ecto.Schema.llm_description/0","ref":"Mentor.Ecto.Schema.html#c:llm_description/0"},{"type":"function","doc":"Validates the given data against the specified schema by applying the schema's `changeset/2` function.","title":"Mentor.Ecto.Schema.validate/2","ref":"Mentor.Ecto.Schema.html#validate/2"},{"type":"function","doc":"- `schema`: The schema module implementing the `Mentor.Ecto.Schema` behaviour.\n- `data`: A map containing the data to be validated.","title":"Parameters - Mentor.Ecto.Schema.validate/2","ref":"Mentor.Ecto.Schema.html#validate/2-parameters"},{"type":"function","doc":"- `{:ok, struct}`: If the data is valid and conforms to the schema.\n- `{:error, changeset}`: If the data is invalid, returns the changeset with errors.","title":"Returns - Mentor.Ecto.Schema.validate/2","ref":"Mentor.Ecto.Schema.html#validate/2-returns"},{"type":"function","doc":"iex> data = %{\"name\" => \"Alice\", \"age\" => 30}\n    iex> Mentor.Ecto.Schema.validate(MyApp.Schema, data)\n    {:ok, %MyApp.Schema{name: \"Alice\", age: 30}}\n\n    iex> invalid_data = %{\"name\" => \"Alice\", \"age\" => 150}\n    iex> Mentor.Ecto.Schema.validate(MyApp.Schema, invalid_data)\n    {:error, %Ecto.Changeset{errors: [age: {\"must be less than 100\", [validation: :number, less_than: 100]}]}}","title":"Examples - Mentor.Ecto.Schema.validate/2","ref":"Mentor.Ecto.Schema.html#validate/2-examples"},{"type":"behaviour","doc":"`Mentor.Ecto.Type` is a behaviour that lets your implement your own custom `Ecto.Type`\n  that works natively with Instructor.","title":"Mentor.Ecto.Type","ref":"Mentor.Ecto.Type.html"},{"type":"behaviour","doc":"```elixir\ndefmodule MyCustomType do\n  use Ecto.Type\n  use Mentor.Ecto.Type\n\n  # ... See `Ecto.Type` for implementation details\n\n  def to_json_schema do\n    %{\n      type: \"string\",\n      format: \"email\"\n    }\n  end\nend\n```","title":"Example - Mentor.Ecto.Type","ref":"Mentor.Ecto.Type.html#module-example"},{"type":"callback","doc":"","title":"Mentor.Ecto.Type.to_json_schema/0","ref":"Mentor.Ecto.Type.html#c:to_json_schema/0"},{"type":"behaviour","doc":"Defines the behaviour for HTTP client adapters within the Mentor framework.\n\nThis module specifies a set of callbacks that any HTTP client adapter must implement to integrate seamlessly with Mentor. By adhering to this behaviour, developers can create custom adapters that handle HTTP requests and responses according to their specific requirements.","title":"Mentor.HTTPClient.Adapter","ref":"Mentor.HTTPClient.Adapter.html"},{"type":"behaviour","doc":"- `request/2`: Initiates an HTTP request based on the provided `Mentor` struct and options.\n- `stream/2` (optional): Initiates a streaming HTTP request.\n- `stream/3` (optional): Initiates a streaming HTTP request with a callback for processing each response chunk.","title":"Callbacks - Mentor.HTTPClient.Adapter","ref":"Mentor.HTTPClient.Adapter.html#module-callbacks"},{"type":"behaviour","doc":"- `status`: Represents the HTTP status code as an integer.\n- `headers`: A list of tuples containing header names and values.\n- `body`: The response body as a binary.\n- `response`: The HTTP response, typically represented by `Finch.Response.t()`.","title":"Types - Mentor.HTTPClient.Adapter","ref":"Mentor.HTTPClient.Adapter.html#module-types"},{"type":"behaviour","doc":"To implement a custom HTTP client adapter, define a module that implements the `Mentor.HTTPClient.Adapter` behaviour:\n\n    defmodule MyApp.HTTPClient.CustomAdapter do\n      @behaviour Mentor.HTTPClient.Adapter\n\n      @impl true\n      def request(%Mentor{} = mentor, opts) do\n        # Implementation for processing the mentor struct and options\n      end\n\n      @impl true\n      def stream(%Mentor{} = mentor, opts) do\n        # Optional implementation for streaming requests\n      end\n\n      @impl true\n      def stream(%Mentor{} = mentor, on_response, opts) do\n        # Optional implementation for streaming requests with a response callback\n      end\n    end\n\nBy implementing these callbacks, your custom adapter can be utilized within the Mentor framework to handle HTTP interactions.","title":"Usage - Mentor.HTTPClient.Adapter","ref":"Mentor.HTTPClient.Adapter.html#module-usage"},{"type":"behaviour","doc":"- **Optional Callbacks**: The `stream/2` and `stream/3` callbacks are optional. If your adapter does not support streaming, you may omit these implementations.\n- **Error Handling**: Ensure that your adapter appropriately handles errors and returns `{:error, reason}` tuples when necessary.\n- **Integration**: Custom adapters allow Mentor to interface with various HTTP clients, such as [Mint](https://github.com/elixir-mint/mint), [Finch](https://github.com/sneako/finch), or [Tesla](https://github.com/elixir-tesla/tesla), providing flexibility in choosing the underlying HTTP client library.\n\nBy following this behaviour, developers can create flexible and interchangeable HTTP client adapters that integrate seamlessly with the Mentor framework.","title":"Considerations - Mentor.HTTPClient.Adapter","ref":"Mentor.HTTPClient.Adapter.html#module-considerations"},{"type":"type","doc":"","title":"Mentor.HTTPClient.Adapter.body/0","ref":"Mentor.HTTPClient.Adapter.html#t:body/0"},{"type":"type","doc":"","title":"Mentor.HTTPClient.Adapter.headers/0","ref":"Mentor.HTTPClient.Adapter.html#t:headers/0"},{"type":"function","doc":"Checks if the given module implements the `Mentor.HTTPClient.Adapter` behaviour.","title":"Mentor.HTTPClient.Adapter.impl_by?/1","ref":"Mentor.HTTPClient.Adapter.html#impl_by?/1"},{"type":"function","doc":"- `module`: The module to be checked.","title":"Parameters - Mentor.HTTPClient.Adapter.impl_by?/1","ref":"Mentor.HTTPClient.Adapter.html#impl_by?/1-parameters"},{"type":"function","doc":"- `true` if the module implements the behaviour.\n- `false` otherwise.","title":"Returns - Mentor.HTTPClient.Adapter.impl_by?/1","ref":"Mentor.HTTPClient.Adapter.html#impl_by?/1-returns"},{"type":"function","doc":"iex> Mentor.HTTPClient.Adapter.impl_by?(MyApp.HTTPClient.CustomAdapter)\n    true\n\n    iex> Mentor.HTTPClient.Adapter.impl_by?(String)\n    false","title":"Examples - Mentor.HTTPClient.Adapter.impl_by?/1","ref":"Mentor.HTTPClient.Adapter.html#impl_by?/1-examples"},{"type":"callback","doc":"","title":"Mentor.HTTPClient.Adapter.request/4","ref":"Mentor.HTTPClient.Adapter.html#c:request/4"},{"type":"type","doc":"","title":"Mentor.HTTPClient.Adapter.request_opts/0","ref":"Mentor.HTTPClient.Adapter.html#t:request_opts/0"},{"type":"type","doc":"","title":"Mentor.HTTPClient.Adapter.status/0","ref":"Mentor.HTTPClient.Adapter.html#t:status/0"},{"type":"type","doc":"","title":"Mentor.HTTPClient.Adapter.url/0","ref":"Mentor.HTTPClient.Adapter.html#t:url/0"},{"type":"module","doc":"An HTTP client adapter for the Mentor framework, utilizing the Finch library for efficient and performant HTTP requests.\n\nThis module implements the `Mentor.HTTPClient.Adapter` behaviour, providing the necessary functions to handle HTTP requests within the Mentor framework using Finch. Finch is an HTTP client focused on performance, built on top of Mint and NimblePool. [oai_citation_attribution:1‡GitHub](https://github.com/sneako/finch?utm_source=chatgpt.com)","title":"Mentor.HTTPClient.Finch","ref":"Mentor.HTTPClient.Finch.html"},{"type":"module","doc":"To use this adapter, ensure that Finch is started and configured appropriately in your application. Typically, you'll add Finch to your supervision tree:\n\n    children = [\n      {Finch, name: Mentor.Finch}\n    ]\n\nWith Finch running, the `Mentor.HTTPClient.Finch` module can handle HTTP requests as defined by the `Mentor.HTTPClient.Adapter` behaviour.","title":"Usage - Mentor.HTTPClient.Finch","ref":"Mentor.HTTPClient.Finch.html#module-usage"},{"type":"module","doc":"- **Finch Configuration**: Ensure that Finch is properly configured and started in your application's supervision tree to handle HTTP requests effectively.\n\n- **Request Options**: The `request/2` function accepts a keyword list of options that are passed to Finch's `request/3` function. Refer to Finch's documentation for details on available options and their effects. [oai_citation_attribution:0‡HexDocs](https://hexdocs.pm/finch/Finch.html?utm_source=chatgpt.com)\n\n- **Error Handling**: Implement appropriate error handling in your application to manage potential issues during HTTP requests, such as network errors or unexpected responses.\n\nBy leveraging Finch, this adapter provides a high-performance HTTP client solution within the Mentor framework, suitable for applications requiring efficient HTTP interactions.","title":"Considerations - Mentor.HTTPClient.Finch","ref":"Mentor.HTTPClient.Finch.html#module-considerations"},{"type":"behaviour","doc":"Defines the behaviour for Large Language Model (LLM) adapters within the Mentor framework.\n\nThis module specifies the required callbacks that any LLM adapter must implement to integrate seamlessly with Mentor. It also provides utility functions to assist in verifying and using these adapters.","title":"Mentor.LLM.Adapter","ref":"Mentor.LLM.Adapter.html"},{"type":"behaviour","doc":"- `complete/1`: Processes a `Mentor` struct and returns a result.\n- `complete!/1`: Similar to `complete/1` but raises an exception in case of an error.","title":"Callbacks - Mentor.LLM.Adapter","ref":"Mentor.LLM.Adapter.html#module-callbacks"},{"type":"behaviour","doc":"To create a custom LLM adapter, define a module that implements the `Mentor.LLM.Adapter` behaviour:\n\n    defmodule MyApp.LLM.CustomAdapter do\n      use Mentor.LLM.Adapter\n\n      @impl true\n      def complete(%Mentor{} = mentor) do\n        # Implementation for processing the mentor struct\n      end\n    end\n\n`complete!/1` is automatic derived and implemented for you, but you can safely override it if needed.\n\nBy implementing these callbacks, your custom adapter can be utilized within the Mentor framework to handle LLM interactions.","title":"Usage - Mentor.LLM.Adapter","ref":"Mentor.LLM.Adapter.html#module-usage"},{"type":"macro","doc":"Macro to be used when implementing the `Mentor.LLM.Adapter` behaviour.\n\nThis macro sets up the necessary boilerplate for defining an adapter, including the `@behaviour` attribute and a default implementation for `complete!/1` that raises an exception on error.","title":"Mentor.LLM.Adapter.__using__/1","ref":"Mentor.LLM.Adapter.html#__using__/1"},{"type":"macro","doc":"defmodule MyApp.LLM.CustomAdapter do\n      use Mentor.LLM.Adapter\n\n      @impl true\n      def complete(%Mentor{} = mentor) do\n        # Implementation for processing the mentor struct\n      end\n    end\n\nBy using this macro, your module declares adherence to the `Mentor.LLM.Adapter` behaviour and provides a default implementation for `complete!/1`.","title":"Usage - Mentor.LLM.Adapter.__using__/1","ref":"Mentor.LLM.Adapter.html#__using__/1-usage"},{"type":"callback","doc":"Callback to process a `Mentor` struct and return a result.","title":"Mentor.LLM.Adapter.complete/1","ref":"Mentor.LLM.Adapter.html#c:complete/1"},{"type":"callback","doc":"- `mentor`: A `Mentor` struct containing the state and configuration for the LLM interaction.","title":"Parameters - Mentor.LLM.Adapter.complete/1","ref":"Mentor.LLM.Adapter.html#c:complete/1-parameters"},{"type":"callback","doc":"- `{:ok, result}`: On successful processing.\n- `{:error, reason}`: If an error occurs during processing.\n\nImplement this callback in your adapter module to define how the LLM interaction should be handled.","title":"Returns - Mentor.LLM.Adapter.complete/1","ref":"Mentor.LLM.Adapter.html#c:complete/1-returns"},{"type":"callback","doc":"","title":"Mentor.LLM.Adapter.complete!/1","ref":"Mentor.LLM.Adapter.html#c:complete!/1"},{"type":"function","doc":"Checks if the given module implements the `Mentor.LLM.Adapter` behaviour.","title":"Mentor.LLM.Adapter.impl_by?/1","ref":"Mentor.LLM.Adapter.html#impl_by?/1"},{"type":"function","doc":"- `module`: The module to be checked.","title":"Parameters - Mentor.LLM.Adapter.impl_by?/1","ref":"Mentor.LLM.Adapter.html#impl_by?/1-parameters"},{"type":"function","doc":"- `true` if the module implements the behaviour.\n- `false` otherwise.","title":"Returns - Mentor.LLM.Adapter.impl_by?/1","ref":"Mentor.LLM.Adapter.html#impl_by?/1-returns"},{"type":"function","doc":"iex> Mentor.LLM.Adapter.impl_by?(MyApp.LLM.CustomAdapter)\n    true\n\n    iex> Mentor.LLM.Adapter.impl_by?(String)\n    false","title":"Examples - Mentor.LLM.Adapter.impl_by?/1","ref":"Mentor.LLM.Adapter.html#impl_by?/1-examples"},{"type":"module","doc":"An adapter for integrating Google's Gemini language models with the Mentor framework.\n\nThis module implements the `Mentor.LLM.Adapter` behaviour, enabling communication between Mentor and Google Generative AI API. It facilitates sending prompts and receiving responses, ensuring compatibility with Mentor's expected data structures.","title":"Mentor.LLM.Adapters.Gemini","ref":"Mentor.LLM.Adapters.Gemini.html"},{"type":"module","doc":"* `:url` (`t:String.t/0`) - Base API endpoint to use for sending requests The default value is `\"https://generativelanguage.googleapis.com/v1beta/models\"`.\n\n* `:api_key` (`t:String.t/0`) - Required. Google Generative AI API key\n\n* `:model` - Required. The Gemini model to query on, known models are: `[\"gemini-2.0-pro\", \"gemini-2.0-pro-latest\", \"gemini-2.0-pro-vision\",\n   \"gemini-2.0-flash\", \"gemini-2.0-flash-lite\", \"gemini-2.0-flash-latest\",\n   \"gemini-2.0-flash-vision\", \"gemini-1.5-pro\", \"gemini-1.5-pro-latest\",\n   \"gemini-1.5-flash\"]`\n\n* `:temperature` (`t:float/0`) - What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. The default value is `1.0`.\n\n* `:http_options` (`t:keyword/0`) - The default value is `[]`.\n\n  * `:pool_timeout` (`t:integer/0`)\n\n  * `:receive_timeout` (`t:integer/0`)\n\n  * `:request_timeout` (`t:integer/0`)","title":"Options - Mentor.LLM.Adapters.Gemini","ref":"Mentor.LLM.Adapters.Gemini.html#module-options"},{"type":"module","doc":"To utilize this adapter, configure your `Mentor` instance with the appropriate options:\n\n    config = [\n      api_key: System.get_env(\"GEMINI_API_KEY\"),\n      model: \"gemini-2.0-pro\"\n    ]\n\n    mentor = Mentor.start_chat_with!(Mentor.LLM.Adapters.Gemini, adapter_config: config)","title":"Usage - Mentor.LLM.Adapters.Gemini","ref":"Mentor.LLM.Adapters.Gemini.html#module-usage"},{"type":"module","doc":"The adapter supports multimodal inputs using the Gemini API format directly:\n\n    Mentor.append_message(%{\n      role: \"user\",\n      content: [\n        %{\n          type: \"text\",\n          text: \"Extract information from this image.\"\n        },\n        %{\n          type: \"image_base64\",\n          data: \"base64_encoded_data\",\n          mime_type: \"image/jpeg\"\n        }\n      ]\n    })\n\nThe adapter passes these formats directly to Gemini with minimal transformation, allowing you to\nuse any content format supported by the Gemini API.","title":"Multimodal Support - Mentor.LLM.Adapters.Gemini","ref":"Mentor.LLM.Adapters.Gemini.html#module-multimodal-support"},{"type":"module","doc":"- **API Key Security**: Ensure your Google API key is stored securely and not exposed in your codebase.\n- **Model Availability**: Verify that the specified model is available and suitable for your use case. Refer to Google's official documentation for the most up-to-date list of models and their capabilities.\n- **Vision Models**: For image processing, use vision-capable models.\n- **Error Handling**: The `complete/1` function returns `{:ok, response}` on success or `{:error, reason}` on failure. Implement appropriate error handling in your application to manage these scenarios.","title":"Considerations - Mentor.LLM.Adapters.Gemini","ref":"Mentor.LLM.Adapters.Gemini.html#module-considerations"},{"type":"function","doc":"Same as `complete/1` but raises an exception in case of error","title":"Mentor.LLM.Adapters.Gemini.complete!/1","ref":"Mentor.LLM.Adapters.Gemini.html#complete!/1"},{"type":"module","doc":"An adapter for integrating OpenAI's language models with the Mentor framework.\n\nThis module implements the `Mentor.LLM.Adapter` behaviour, enabling communication between Mentor and OpenAI's API. It facilitates sending prompts and receiving responses, ensuring compatibility with Mentor's expected data structures.","title":"Mentor.LLM.Adapters.OpenAI","ref":"Mentor.LLM.Adapters.OpenAI.html"},{"type":"module","doc":"* `:url` (`t:String.t/0`) - API endpoint to use for sending requests The default value is `\"https://api.openai.com/v1/chat/completions\"`.\n\n* `:api_key` (`t:String.t/0`) - Required. OpenAI API key\n\n* `:model` - Required. The OpenAI model to query on, known models are: `[\"gpt-4o\", \"gpt-4o-mini\", \"o1\", \"o1-mini\", \"o1-preview\", \"chatgpt-4o-latest\",\n   \"gpt-4o-realtime-preview\", \"gpt-4o-mini-realtime-preview\",\n   \"gpt-4o-audio-preview\"]`\n\n* `:temperature` (`t:float/0`) - What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic. The default value is `1.0`.\n\n* `:http_options` (`t:keyword/0`) - The default value is `[]`.\n\n  * `:pool_timeout` (`t:integer/0`)\n\n  * `:receive_timeout` (`t:integer/0`)\n\n  * `:request_timeout` (`t:integer/0`)","title":"Options - Mentor.LLM.Adapters.OpenAI","ref":"Mentor.LLM.Adapters.OpenAI.html#module-options"},{"type":"module","doc":"To utilize this adapter, configure your `Mentor` instance with the appropriate options:\n\n    config = [\n      url: \"https://api.openai.com/v1/chat/completions\",\n      api_key: System.get_env(\"OPENAI_API_KEY\"),\n      model: \"gpt-4o\"\n    ]\n\n    mentor = Mentor.start_chat_with!(Mentor.LLM.Adapters.OpenAI, adapter_config: config)","title":"Usage - Mentor.LLM.Adapters.OpenAI","ref":"Mentor.LLM.Adapters.OpenAI.html#module-usage"},{"type":"module","doc":"- **API Key Security**: Ensure your OpenAI API key is stored securely and not exposed in your codebase.\n- **Model Availability**: Verify that the specified model is available and suitable for your use case. Refer to OpenAI's official documentation for the most up-to-date list of models and their capabilities.\n- **Error Handling**: The `complete/1` function returns `{:ok, response}` on success or `{:error, reason}` on failure. Implement appropriate error handling in your application to manage these scenarios.\n\nBy adhering to the `Mentor.LLM.Adapter` behaviour, this module ensures seamless integration with OpenAI's API, allowing for efficient and effective language model interactions within the Mentor framework.","title":"Considerations - Mentor.LLM.Adapters.OpenAI","ref":"Mentor.LLM.Adapters.OpenAI.html#module-considerations"},{"type":"function","doc":"Same as `complete/1` but raises an exception in case of error","title":"Mentor.LLM.Adapters.OpenAI.complete!/1","ref":"Mentor.LLM.Adapters.OpenAI.html#complete!/1"},{"type":"module","doc":"Responsible for verifying if all schema fields are present in the documentation either\nin the `@moduledoc` attribute or as the return of the `llm_description/0` callback.","title":"Mentor.Parser","ref":"Mentor.Parser.html"},{"type":"function","doc":"Verifies the provided markdown to extract field names that are missing descriptions.","title":"Mentor.Parser.run/2","ref":"Mentor.Parser.html#run/2"},{"type":"function","doc":"- `markdown`: The markdown string to parse.\n- `expected_fields`: A list of atoms or strings representing the expected field names.","title":"Parameters - Mentor.Parser.run/2","ref":"Mentor.Parser.html#run/2-parameters"},{"type":"function","doc":"iex> markdown = \"\"\"\n    ...>","title":"Examples - Mentor.Parser.run/2","ref":"Mentor.Parser.html#run/2-examples"},{"type":"function","doc":"...>\n    ...> - `name`: The user's name.\n    ...> - `age`: The user's age.\n    ...> - `offices_held`:\n    ...>   - `office`: The political office name\n    ...>   - `from_date`: Start date\n    ...> \"\"\"\n    iex> Mentor.Parser.run(markdown, [\"name\", \"age\", \"offices_held.office\", \"offices_held.from_date\"])\n    []\n\n    iex> markdown = \"\"\"\n    ...>","title":"Fields - Mentor.Parser.run/2","ref":"Mentor.Parser.html#run/2-fields"},{"type":"function","doc":"...>\n    ...> - name: The user's name.\n    ...> - offices_held:\n    ...>   - office: The political office\n    ...> \"\"\"\n    iex> Mentor.Parser.run(markdown, [:name, \"offices_held.office\", \"offices_held.from_date\"])\n    [\"offices_held.from_date\"]","title":"Fields - Mentor.Parser.run/2","ref":"Mentor.Parser.html#run/2-fields"},{"type":"behaviour","doc":"Defines the `Mentor.Schema` behaviour for integrating various data structures\nwith the Mentor framework. This behaviour allows for the creation of schema\nadapters, enabling Mentor to work seamlessly with different data representations\nsuch as `Ecto` schemas, raw structs, and maps.\n\nModules implementing this behaviour are required to define the following callbacks:\n\n  * `c:definition/1` - Specifies the schema definition, detailing the fields and their types.\n  * `c:validate/2` - Validates the provided data against the schema, returning the validated structure or an error term.\n\nAdditionally, this module provides functions to parse and validate field documentation, ensuring that all fields are properly documented.","title":"Mentor.Schema","ref":"Mentor.Schema.html"},{"type":"callback","doc":"","title":"Mentor.Schema.definition/1","ref":"Mentor.Schema.html#c:definition/1"},{"type":"type","doc":"","title":"Mentor.Schema.field/0","ref":"Mentor.Schema.html#t:field/0"},{"type":"function","doc":"Raises a compile-time error indicating which fields are missing documentation\nin the `@moduledoc` attribute or the `llm_description/0` callback.","title":"Mentor.Schema.missing_documentation!/2","ref":"Mentor.Schema.html#missing_documentation!/2"},{"type":"function","doc":"* `env` - The current macro environment.\n  * `fields` - A list of fields missing documentation.","title":"Parameters - Mentor.Schema.missing_documentation!/2","ref":"Mentor.Schema.html#missing_documentation!/2-parameters"},{"type":"function","doc":"iex> Mentor.Schema.missing_documentation!(__ENV__, [:name, :age])\n    ** (CompileError) The following fields are missing documentation either in `@moduledoc` attribute or `llm_description/0` callback: [:name, :age]","title":"Examples - Mentor.Schema.missing_documentation!/2","ref":"Mentor.Schema.html#missing_documentation!/2-examples"},{"type":"function","doc":"Parses the given documentation string to ensure that all fields in the schema\nare properly documented. If any fields are missing documentation, a compile-time\nerror is raised.","title":"Mentor.Schema.parse_llm_description!/3","ref":"Mentor.Schema.html#parse_llm_description!/3"},{"type":"function","doc":"* `doc` - The documentation string to parse.\n  * `schema` - A list of fields expected in the schema.\n  * `env` - The current macro environment.","title":"Parameters - Mentor.Schema.parse_llm_description!/3","ref":"Mentor.Schema.html#parse_llm_description!/3-parameters"},{"type":"function","doc":"iex> Mentor.Schema.parse_llm_description!(doc_string, [:name, :age], __ENV__)\n    :ok","title":"Examples - Mentor.Schema.parse_llm_description!/3","ref":"Mentor.Schema.html#parse_llm_description!/3-examples"},{"type":"type","doc":"","title":"Mentor.Schema.source/0","ref":"Mentor.Schema.html#t:source/0"},{"type":"type","doc":"","title":"Mentor.Schema.t/0","ref":"Mentor.Schema.html#t:t/0"},{"type":"callback","doc":"","title":"Mentor.Schema.validate/2","ref":"Mentor.Schema.html#c:validate/2"},{"type":"module","doc":"","title":"NumberSeries","ref":"NumberSeries.html"},{"type":"module","doc":"- `series`: an array of integers","title":"Fields - NumberSeries","ref":"NumberSeries.html#module-fields"},{"type":"module","doc":"A description of Brazilian Politicians and the offices that they held, you can specify only the most relevants held offices in the past 20 years.","title":"Politician","ref":"Politician.html"},{"type":"module","doc":"- `first_name`: Their first name\n- `party`: Theier politic party, the most recent you have knowledge\n- `last_name`: Their last name\n- `offices_held`:\n  - `office`: The name of the political office held by the politician (in lowercase)\n  - `from_date`: When they entered office (YYYY-MM-DD)\n  - `to_date`: The date they left office, if relevant (YYYY-MM-DD or null).","title":"Fields - Politician","ref":"Politician.html#module-fields"},{"type":"module","doc":"","title":"Politician.Office","ref":"Politician.Office.html"},{"type":"module","doc":"","title":"Schema","ref":"Schema.html"},{"type":"extras","doc":"# Mentor\n\n> [!WARNING]\n> This library is under actively development, expect breaking changes, although the main public API is kinda stable\n\n[![mentor version](https://img.shields.io/hexpm/v/mentor.svg)](https://hex.pm/packages/mentor)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/mentor)\n[![Hex Downloads](https://img.shields.io/hexpm/dt/mentor)](https://hex.pm/packages/mentor)\n\nBecause we love composability!\n\nNo magic, but a higher-level API to generate structed output with LLM, based on a \"schema\", it can be:\n- a raw map/struct/data structure (string, tuple and so on) (wip)\n- an `Ecto` schema (embedded, database or schemaless changesets)\n- a `peri` schema definition (wip)","title":"Mentor","ref":"readme.html"},{"type":"extras","doc":"```elixir\ndef deps do\n  [\n    {:mentor, \"~> 0.2\"}\n  ]\nend\n```","title":"Installation - Mentor","ref":"readme.html#installation"},{"type":"extras","doc":"The `mentor` library is useful for coaxing an LLM to return JSON that maps to a schema that you provide, rather than the default unstructured text output. If you define your own validation logic, `mentor` can automatically retry prompts when validation fails (returning natural language error messages to the LLM, to guide it when making corrections).\n\n`mentor` is designed to be used with a variaty of LLM providers like [OpenAI API](https://platform.openai.com/docs/api-reference/chat-completions/create), [llama.cpp](https://github.com/ggerganov/llama.cpp), [Bumblebee](https://github.com/elixir-nx/bumblebee) and so on (check the [LLM.Adapters](#llm-adapters) section) by using an extendable adapter behavior.\n\nAt its simplest, usage with `Ecto` is pretty straightforward:\n\n1. Create an `Ecto` schema, with a native `@moduledoc` string that explains the schema definition to the LLM or define a `llm_description/0` callback to return it.\n2. Use the `Mentor.Ecto.Schema` macro to validate and fetch the documentation prompt and enforce callbacks.\n3. Define an usual `changeset/2` function on the schema.\n4. Construct a `Mentor` and pass it to `Mentor.complete/1` to generate the structured output.\n\n```elixir\ndefmodule SpamPrediction do\n  @moduledoc \"\"\"","title":"Usage - Mentor","ref":"readme.html#usage"},{"type":"extras","doc":"- class: Whether or not the email is spam.\n  - reason: A short, less than 10 word rationalization for the classification.\n  - score: A confidence score between 0.0 and 1.0 for the classification.\n  \"\"\"\n\n  use Ecto.Schema\n  use Mentor.Ecto.Schema\n\n  import Ecto.Changeset\n\n  @primary_key false\n  embedded_schema do\n    field :class, Ecto.Enum, values: [:spam, :not_spam]\n    field :reason, :string\n    field :score, :float\n  end\n\n  @impl true\n  def changeset(%__MODULE__{} = source, %{} = attrs) do\n    source\n    |> cast(attrs, [:class, :reason, :score])\n    |> validate_number(:score, greater_than_or_equal_to: 0.0, less_than_or_equal_to: 1.0)\n  end\n\n  # this can be in another module and the function name doesn't matter\n  def instruct(text) when is_binary(text) do\n    Mentor.start_chat_with!(Mentor.LLM.Adapters.OpenAI,\n      schema: __MODULE__,\n      max_retries: 2 # defaults to 3\n    )\n    # append how many custom messages you want\n    |> Mentor.append_message(%{\n      role: \"user\",\n      content: \"Classify the following email: #{text}\"\n    })\n    # pass specific config to the adapter\n    |> Mentor.configure_adapter(api_key: System.fetch_env!(\"OPENAI_API_KEY\"), model: \"gpt-4o-mini\")\n    # you can also overwrite the initial prompt\n    |> Mentor.overwrite_initial_prompt(\"\"\"\n    Your purpose is to classify customer support emails as either spam or not.\n    This is for a clothing retail business.\n    They sell all types of clothing.\n    \"\"\")\n    # trigger the instruction to be completed (aka sent it to the LLM)\n    # since all above steps are lazy\n    |> Mentor.complete()\n  end\nend\n\nSpamPrediction.instruct(\"\"\"\nHello I am a Nigerian prince and I would like to send you money\n\"\"\")\n# => {:ok, %SpamPrediction{class: :spam, reason: \"Nigerian prince email scam\", score: 0.98}}\n```\n\nCheck out our [Quickstart Guide](https://hexdocs.pm/mentor/quickstart.html) for more code snippets that you can run locally (in Livebook). Or, to get a better idea of the thinking behind Instructor, read more about our [Philosophy & Motivations](https://hexdocs.pm/mentor/philosophy.html).","title":"Field Descriptions: - Mentor","ref":"readme.html#field-descriptions"},{"type":"extras","doc":"- [OpenAI text generation docs](https://platform.openai.com/docs/guides/text-generation)\n- [OpenAI structured outputs docs](https://platform.openai.com/docs/guides/structured-outputs)\n- [OpenAI input formatting cookbook](https://cookbook.openai.com/examples/how_to_format_inputs_to_chatgpt_models)\n- [Anthropic Message format](https://docs.anthropic.com/en/api/messages#body-messages)\n- [Google Gemini structured outputs docs](https://ai.google.dev/gemini-api/docs/structured-output?lang=rest)","title":"References - Mentor","ref":"readme.html#references"},{"type":"extras","doc":"- [instructor_ex](https://hexdocs.pm/instructor)\n- [instructor_lite](https://hexdocs.pm/instructor_lite)","title":"Spiritual inspirations - Mentor","ref":"readme.html#spiritual-inspirations"},{"type":"extras","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\n## [0.2.3]  - 2025-04-16","title":"Changelog","ref":"changelog.html"},{"type":"extras","doc":"- Added Google Gemini adapter for structured output support\n\n## [0.2.2] - 2025-04-14","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Minor bug fixes and improvements\n\n## [0.2.1] - 2025-02-15","title":"Changes - Changelog","ref":"changelog.html#changes"},{"type":"extras","doc":"- Fixed minor bugs in OpenAI adapter\n- Improved error handling\n\n## [0.2.0] - 2025-01-28","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Added `configure_http_client/3` function to provide a more flexible way to configure HTTP clients\n- Added type specifications for all public functions","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- **Breaking**: Removed `adapter_config` from `start_chat_with!/2` options. Configuration should now be done through `configure_adapter/2`\n- Improved error handling for HTTP client validation\n- Enhanced error handling for better insight into validation failures\n- Restructured the codebase for better maintainability","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fixed Dialyzer warnings across the codebase\n- Fixed type specifications to be more accurate and complete\n\n## [0.1.3] - 2025-01-22","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Added HTTP options configuration support for OpenAI adapter\n- Added configurable timeouts for HTTP requests","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Improved HTTP client configuration handling\n- Enhanced documentation and examples\n\n## [0.1.2] - 2025-01-22","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Added temperature configuration option for OpenAI adapter\n- Added proper HTTP timeout configurations","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Fixed module loading issues with `Code.ensure_loaded?/1`\n- Fixed custom type handling in JSON Schema generation","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Improved error handling in custom type implementations\n- Enhanced documentation for custom types\n\n## [0.1.1] - 2025-01-21","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Added support for custom `llm_description/0` callback\n- Added new parser implementation for better field documentation validation\n- Added comprehensive documentation and examples\n- Added evaluation modules for testing and demonstration","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"- Simplified the parser implementation\n- Improved documentation parsing and validation\n- Enhanced error messages for missing field documentation","title":"Changed - Changelog","ref":"changelog.html#changed"},{"type":"extras","doc":"- Fixed documentation validation for nested fields\n- Fixed schema field parsing\n\nAll these changes follow semantic versioning principles, where:\n- 0.2.0 includes breaking changes to the API\n- 0.1.1 through 0.1.3 add new features and fixes in a backward-compatible way\n\n## [0.1.0] - 2025-01-20","title":"Fixed - Changelog","ref":"changelog.html#fixed"},{"type":"extras","doc":"- Initial release of Mentor.\n  - Provides a high-level API to generate structured outputs based on various schemas, including raw maps, structs, and `Ecto` schemas.\n  - Implements the `Mentor.LLM.Adapter` behaviour for integrating with different Large Language Models (LLMs).\n  - Includes an adapter for OpenAI's language models.\n  - Supports defining and validating data structures using the `Mentor.Schema` protocol.\n  - Offers basic debugging capabilities.\n\n> Note: This changelog follows the principles of [Keep a Changelog](https://keepachangelog.com/en/1.0.0/).","title":"Added - Changelog","ref":"changelog.html#added"},{"type":"extras","doc":"# Philosophy & Motivations\n\n`mentor` is simple library that allows you to maximally leverage large language models (LLMs).\nIt acts as your bridge between [Karpathy's Software 1.0 and Software 2.0](https://karpathy.medium.com/software-2-0-a64152b37c35).\n\nSoftware 1.0 is the world we're all used to. \nIt's datastructures, databases, algorithms, code.\n\nSoftware 2.0 on the other hand is neural networks.\nThey're models to problems for which it's easier to gather examples of desired behavior than describe how to produce it.","title":"Philosophy & Motivations","ref":"philosofy.html"},{"type":"extras","doc":"Software 2.0, as it exists today operates in the modality of text, images, and audio.\nAll modalities that have been traditionally very difficult to deal with in Software 1.0.\n\nWe now have these things called LLMs.\nFor the first time we can solve non-trivial `f(text) -> text` problems.\n\n```\n\n  llm(\"Recipe for a banana smoothie\") -> \"\"\"\n    A banana smoothie is a delicious and nutritious treat that's quite easy to make. Here's a simple recipe for you:\n\n    Ingredients:\n\n    2 ripe bananas\n    1 cup milk (you can use almond, soy, or cow's milk)\n    1/2 cup Greek yogurt (optional, for extra creaminess)\n    1-2 tablespoons honey or maple syrup (adjust according to taste)\n    A pinch of cinnamon (optional)\n    Ice cubes (optional, for a colder smoothie)\n    Instructions:\n\n    Peel the bananas and place them in a blender.\n    Add the milk, Greek yogurt (if using), honey/maple syrup, and cinnamon.\n    Add a few ice cubes if you prefer a colder smoothie.\n    Blend until smooth and creamy. If the smoothie is too thick, you can add a little more milk to reach your desired consistency.\n    Taste and adjust the sweetness if necessary.\n    Pour into glasses and serve immediately.\n\n    Enjoy your banana smoothie!\n  \"\"\"\n\n```\n\nHowever from a Software 1.0 perspective, this isn't terribly useful.\nRemember, Software 1.0 doesn't have robust, non trivial semantic text processing.\nWe can pretty easily create the function's arguments, but are you sure you can robustly parse the output? Unlikely.","title":"A New Modality - Philosophy & Motivations","ref":"philosofy.html#a-new-modality"},{"type":"extras","doc":"Over the past year you've probably noticed that every LLM company produces some variation of a chatbot -\nChat with GPT, Chat with your PDF, Chat with your AI Girlfriend.\n\nThis makes sense, when you think about it.\nWe're really good at creating strings in Software 1.0 from structured data, but we're terrible at creating structured data from strings.\nNor are we any good any good at interpreting the semantic meaning of strings.\n\nTherefore, it only makes sense to design experiences where we present them to the user for interpretation.\nAlas, our ubiquitous chatbot interface.\n\nBut it doesn't have to be that way. \nWhat if we can get our LLMs to respond with structured data?\nThat'd give us the all our domains and ranges covered,\n\n```\n  f(text) -> text\n  f(data_structure |> to_string) -> text\n  f(data_structure |> to_string) -> data_structure\n```\n\nWith such a capability, you would have full bidirectional interoperability between software 1.0 and software 2.0. \nThat's what Instructor provides.","title":"LLMs aren't only for chatbots - Philosophy & Motivations","ref":"philosofy.html#llms-aren-t-only-for-chatbots"},{"type":"extras","doc":"Now to do this we need to give the LLM a schema that it has to conform to in its output text. \nIf we can guarantee that it outputs JSON that matches this schema, we can treat it as any other user data we're used to in Elixir.\n\nThat's why, we figure, let's just build this on top of Ecto. It's already in your app after all.\nYou already know how to use it.\nIt already provides a robust validations API.\nLet's just treat the LLM as if it's one of your users, submitting data through a form.\n\n```elixir\ndefmodule Recipe do\n  @moduledoc \"\"\"\n  Our AI generated delicious recipe.","title":"It's just Elixir baby - Philosophy & Motivations","ref":"philosofy.html#it-s-just-elixir-baby"},{"type":"extras","doc":"- `title`: the title of our recipe\n  - `cook_time`: the recipe cook time\n  - `steps`: a list of descriptive string steps for the recipe\n  - `ingredients`: a list of the recipe ingredients\n    - `name`: the name of the ingredient\n    - `quantity`: the amount of the ingredient\n    - `unit`: the unit of the amoun of the ingredient\n  \"\"\"\n\n  use Ecto.Schema\n  use Mentor.Ecto.Schema\n\n  import Ecto.Chageset\n\n  @primary_key false\n  embedded_schema do \n    field :title, :string\n    field :cook_time, :integer\n    field :steps, {:array, :string}\n    embeds_many :ingredients, Ingredients, primary_key: false do\n      field :name, :string\n      field :quantity, :decimal\n      field :unit, :string\n    end\n  end\n\n  @impl true\n  def changeset(%__MODULE__{} = recipe, %{} = attrs) do\n    recipe\n    |> cast(attrs, [:title, :cook_time, :steps])\n    |> validate_required([:title, :cook_time, :steps])\n    |> cast_embed(:ingredients, required: true, with: &ingredient_changeset/2)\n  end\n\n  defp ingredient_changeset(%{} = ingredient, %{} = attrs) do\n    ingredient\n    |> cast(attrs, [:name, :quantity, :unit])\n    |> validate_required([:name, :quantity, :unit])\n  end\nend\n```\n\nFrom this `mentor` takes your ecto schema, converts it into a JSON schema, and passes it on to the LLM.\nIt uses clever techniques like function calling and BNF grammar sampling to ensure that we get back a result that is exactly matching our Ecto schema.\nAt which point it's all just Ecto code.\nWe cast the fields, we do the validations, and we return the result back to you.\n\n```elixir\nalias Mentor.LLM.Adapters.OpenAI\n\napi_key = System.fetch_env!(\"OPENAI_API_KEY\")\n\nMentor.start_chat_with!(OpenAI, schema: Recipe, max_retries: 2)\n|> Mentor.configure_adapter(api_key: api_key, model: \"gpt-4o-mini\")\n|> Mentor.append_message(%{role: \"user\", content: \"Give me a recipe for a banana smoothie\" })\n|> Mentor.complete()\n\n# => {:ok, %Recipe{title: \"Banana smoothie\", cook_time: 15, steps: [...]}\n```\n\n`mentor` makes `Ecto` is our interchange format between Software 1.0 and 2.0.\nPerfect because `Ecto` is a mainstay in the ecosystem and compatible with so many other libraries.","title":"Fields - Philosophy & Motivations","ref":"philosofy.html#fields"},{"type":"extras","doc":"Now this is a neat capability, but you may be thinking, where can I use this?  \nThis is Virgin Territory. We've never had this capability. It is a new modality in computing.\nIt will require a bit of imagination.\nA good heuristic to start with is, \"wherever I use Ecto, I can use `mentor`.\" \n\n```\n  user -> mentor -> code\n  code -> mentor -> code\n  code -> mentor -> user\n```\n\nThere is a 40Gb LLM file sitting on Huggingface that encodes the entirety of human knowledge, you now have a query interface for it.","title":"Used Anywhere where Ecto is used - Philosophy & Motivations","ref":"philosofy.html#used-anywhere-where-ecto-is-used"},{"type":"extras","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Quickstart\n\n```elixir\nMix.install([:mentor])\n```","title":"Quickstart","ref":"quickstart.html"},{"type":"extras","doc":"`mentor` is a library to do structured prompting with LLMs. While the idea is pretty simple, through this and the other examples you'll realize how powerful a concept this is.\n\n**So first off, what is structure prompting?**\n\nWhat if the LLM returned data conforming to a complicated nested schema that your code knows how to work with? Well, that's structure prompting. It's a way of cohercing the LLM to producing it's response in a known format that your downstream code can handle. In the case of `mentor`, we use `Ecto`, `Peri`, raw structs or even maps to provide those schemas.\n\nSo, without further ado, let's take define a schema and take it for a spin!","title":"Introduction - Quickstart","ref":"quickstart.html#introduction"},{"type":"extras","doc":"```elixir\ndefmodule Politician do\n  @moduledoc \"\"\"\n  A description of Brazilian Politicians and the offices that they held, you can specify only the most relevants held offices in the past 20 years.","title":"Ecto - Quickstart","ref":"quickstart.html#ecto"},{"type":"extras","doc":"- `first_name`: Their first name\n  - `party`: Theier politic party, the most recent you have knowledge\n  - `last_name`: Their last name\n  - `offices_held`:\n    - `office`: The name of the political office held by the politician (in lowercase)\n    - `from_date`: When they entered office (YYYY-MM-DD)\n    - `to_date`: The date they left office, if relevant (YYYY-MM-DD or null).\n  \"\"\"\n\n  use Ecto.Schema\n  use Mentor.Ecto.Schema\n\n  import Ecto.Changeset\n\n  @primary_key false\n  embedded_schema do\n    field :first_name, :string\n    field :last_name, :string\n    field :party, :string\n\n    embeds_many :offices_held, Office, primary_key: false do\n      @offices ~w(president vice_president minister senator deputy governor vice_governor state_secretary state_deputy mayor deputy_mayor city_councilor)a\n\n      field :office, Ecto.Enum, values: @offices\n      field :from_date, :date\n      field :to_date, :date\n    end\n  end\n\n  @impl true\n  def changeset(%__MODULE__{} = politician, %{} = attrs) do\n    politician\n    |> cast(attrs, [:first_name, :last_name, :party])\n    |> validate_required([:first_name, :last_name])\n    |> cast_embed(:offices_held, required: true, with: &offices_held_changeset/2)\n  end\n\n  defp offices_held_changeset(offices, %{} = attrs) do\n    offices\n    |> cast(attrs, [:office, :from_date, :to_date])\n    |> validate_required([:office, :from_date, :to_date])\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Politician, <<70, 79, 82, 49, 0, 0, 17, ...>>,\n [__schema__: 1, __schema__: 1, __schema__: 1, __schema__: 1, __schema__: 2, __schema__: 2, ...]}\n```\n\nGreat, we have our schema describing politicans and the offices they held. Let's notice a few things that may stand out from regular `Ecto` usage. First, since there is no database backing the schema, it doesn't make sense to give it a primary_key. This also makes sense because there is no sensible value for the LLM to respond with.\n\nAlso we use an usual `@moduledoc` on the schema. This isn't just for documentation purposes of the tutorial. `mentor` will take any `@moduledoc` tag and provide it to the LLM. Generally you'll want to use this to provide semantic descriptions of the fields and general context to the LLM to ensure you get the outputs you want. In our case we want to push the LLM to understand that we are only considering Brazilian politicians.\n\nSo, let's try asking the LLM to give us some politicians.\n\n```elixir\nalias Mentor.LLM.Adapters.OpenAI\n\nMentor.start_chat_with!(OpenAI, schema: Politician)\n|> Mentor.configure_adapter(\n  model: \"gpt-4o-mini\",\n  api_key: System.fetch_env!(\"LB_OPENAI_API_KEY\")\n)\n|> Mentor.append_message(%{\n  role: \"user\",\n  content: \"Who won the Brazilian 2022 election and what offices have they held over their career?\"\n})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok,\n %Politician{\n   first_name: \"Luiz Inácio\",\n   last_name: \"Lula da Silva\",\n   party: \"Partido dos Trabalhadores (PT)\",\n   offices_held: [\n     %Politician.Office{\n       office: :president,\n       from_date: ~D[2003-01-01],\n       to_date: ~D[2006-12-31]\n     },\n     %Politician.Office{\n       office: :president,\n       from_date: ~D[2007-01-01],\n       to_date: ~D[2010-12-31]\n     },\n     %Politician.Office{\n       office: :president,\n       from_date: ~D[2023-01-01],\n       to_date: ~D[2026-12-31]\n     }\n   ]\n }}\n```\n\nAmazing, right? Using nothing more than a simple schema, we were able to get structured output from our LLM. The data returned is ready to be processed by our regular Elixir code. `mentor` supports all field types that you can express in schema libraries, like in `Ecto`: including embedded and associated schemas.\n\nIt's almost as if the LLM inputted the data into a Phoenix Form. All the utilities that you use to process that kind of data, you can use to process the outputs of `mentor`.\n\nOne of the superpowers of this is that since we're just using changesets under the hood, you can use the same validations that you would use elsewhere in your app. Let's look at that in the next section.\n\n<!-- livebook:{\"branch_parent_index\":0} -->","title":"Fields - Quickstart","ref":"quickstart.html#fields"},{"type":"extras","doc":"`mentor` leverages `Ecto` changesets to validate the data returned by the LLM. Therefore there is nothing fancy to this API since it uses the usual `changeset/2` function you would already implement.\n\n```elixir\ndefmodule NumberSeries do\n  @moduledoc \"\"\"","title":"Validations - Quickstart","ref":"quickstart.html#validations"},{"type":"extras","doc":"- `series`: an array of integers\n  \"\"\"\n\n  use Ecto.Schema\n  use Mentor.Ecto.Schema\n\n  import Ecto.Changeset\n\n  @primary_key false\n  embedded_schema do\n    field :series, {:array, :integer}\n  end\n\n  @impl true\n  def changeset(%__MODULE__{} = number_series, %{} = attrs) do\n    number_series\n    |> cast(attrs, [:series])\n    |> validate_length(:series, min: 10)\n    |> validate_change(:series, fn\n      field, values ->\n        if Enum.sum(values) |> rem(2) == 0 do\n          []\n        else\n          [{field, \"The sum of the series must be even\"}]\n        end\n    end)\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, NumberSeries, <<70, 79, 82, 49, 0, 0, 18, ...>>, {:changeset, 2}}\n```\n\nIn this albeit contrived example, we're going to get the LLM to return a series of numbers and validate whether it has at least 10 numbers and that the sum of the series is even.\n\nWhen we ask for fewer than ten numbers, `mentor` will return an error tuple with a change set that is invalid.\n\n```elixir\nMentor.start_chat_with!(OpenAI, schema: NumberSeries)\n|> Mentor.configure_adapter(\n  model: \"gpt-4o-mini\",\n  api_key: System.fetch_env!(\"LB_OPENAI_API_KEY\")\n)\n|> Mentor.append_message(%{\n  role: \"user\",\n  content: \"Give me the first 5 integers\"\n})\n|> Mentor.complete\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:error,\n #Ecto.Changeset ,\n   valid?: false,\n   ...\n >}\n```\n\nNow the beauty of this is that since we have human readable errors from our validations, we can just turn around and pass those back into the LLM to get it to fix its own errors.\n\n`mentor` provides a convenience parameter, `max_retries` for you in the initial call which will retry against the validations up to n times.\n\n```elixir\nMentor.start_chat_with!(OpenAI,\n  schema: NumberSeries,\n  max_retries: 10\n)\n|> Mentor.configure_adapter(model: \"gpt-4o-mini\", api_key: System.fetch!(\"LB_OPENAI_API_KEY\"))\n|> Mentor.append_message(%{role: \"user\", content: \"Give some random integers\"})\n|> Mentor.complete\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n10:30:03.764 [debug] Retrying LLM call for NumberSeries:\n\n \"series - The sum of the series must be even\\nseries - should have at least 10 item(s)\"\n\n10:30:04.794 [debug] Retrying LLM call for NumberSeries:\n\n \"series - The sum of the series must be even\"\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok,\n %NumberSeries{series: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]}}\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","title":"Fields - Quickstart","ref":"quickstart.html#fields"},{"type":"extras","doc":"TODO","title":"Record Streaming - Quickstart","ref":"quickstart.html#record-streaming"},{"type":"extras","doc":"`mentor` supports all the `Ecto` types out of the box, but sometimes you need more. And that's why Instructor provides a behavior that you can implement on your own custom Ecto types. All you have to do is implement `to_json_schema/0`.\n\nWhatever you return from this function will be put as the field type. See the [JSONSchema Specification](https://json-schema.org/understanding-json-schema/reference) for more information on what you can put here. Typically you'll see people put `description`, `type`, and maybe `format`.\n\n```elixir\ndefmodule EctoURI do\n  use Ecto.Type\n  use Mentor.Ecto.Type\n\n  def type, do: :map\n\n  # This is it, the rest is for implementing a regular old ecto type.\n  @impl true\n  def to_json_schema do\n    %{\n      type: \"string\",\n      description: \"A valid URL\"\n    }\n  end\n\n  def cast(uri) when is_binary(uri) do\n    {:ok, URI.parse(uri)}\n  end\n\n  def cast(%URI{} = uri), do: {:ok, uri}\n  def cast(_), do: :error\n\n  def load(data) when is_map(data) do\n    data =\n      for {key, val} <- data do\n        {String.to_existing_atom(key), val}\n      end\n\n    {:ok, struct!(URI, data)}\n  end\n\n  def dump(%URI{} = uri), do: {:ok, Map.from_struct(uri)}\n  def dump(_), do: :error\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, EctoURI, <<70, 79, 82, 49, 0, 0, 14, ...>>, {:dump, 1}}\n```\n\n```elixir\nMentor.start_chat_with!(OpenAI,\n  schema: %{url: EctoURI},\n  max_retries: 10\n)\n|> Mentor.configure_adapter(model: \"gpt-4o-mini\", api_key: System.fetch!(\"LB_OPENAI_API_KEY\"))\n|> Mentor.append_message(%{role: \"user\", content: \"Give me the URL for Google\"})\n|> Mentor.complete\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok,\n %{\n   url: %URI{\n     scheme: \"https\",\n     authority: \"www.google.com\",\n     userinfo: nil,\n     host: \"www.google.com\",\n     port: 443,\n     path: nil,\n     query: nil,\n     fragment: nil\n   }\n }}\n```\n\nAnd just like that, you can extend `mentor` to get the LLM to return whatever you want.","title":"Custom Ecto Types - Quickstart","ref":"quickstart.html#custom-ecto-types"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}